@@ -4,10 +4,6 @@ collector <- function(type, ...) {
 
 is.collector <- function(x) inherits(x, "collector")
 
-collector_guess <- function(x, locale = default_locale()) {
-  collectorGuess(x, locale)
-}
-
 #' @export
 print.collector <- function(x, ...) {
   cat("<", class(x)[1], ">\n", sep = "")
@@ -26,6 +22,7 @@ collector_find <- function(name) {
 #' @param x Character vector of elements to parse.
 #' @param collector Column specification.
 #' @inheritParams read_delim
+#' @inheritParams tokenizer_delim
 #' @keywords internal
 #' @export
 #' @examples
@@ -40,22 +37,20 @@ parse_vector <- function(x, collector, na = c("", "NA"), locale = default_locale
   warn_problems(parse_vector_(x, collector, na = na, locale_ = locale))
 }
 
-#' Parse character vectors into typed columns.
+#' Parse character vector in an atomic vector.
 #'
 #' Use \code{parse_} if you have a character vector you want to parse. Use
 #' \code{col_} in conjunction with a \code{read_} function to parse the
 #' values as they're read in.
 #'
-#' @name collector
+#' @name parse_atomic
 #' @param x Character vector of values to parse.
 #' @inheritParams tokenizer_delim
 #' @inheritParams read_delim
-#' @seealso \code{\link{parse_datetime}}, \code{\link{type_convert}} to
-#'   automatically re-parse all character columns in a data frame.
+#' @family parser
 #' @examples
 #' parse_integer(c("1", "2", "3"))
 #' parse_double(c("1", "2", "3.123"))
-#' parse_factor(c("a", "b"), letters)
 #' parse_number("$1,123,456.00")
 #'
 #' # Use locale to override default decimal and grouping marks
@@ -69,130 +64,151 @@ parse_vector <- function(x, collector, na = c("", "NA"), locale = default_locale
 #' parse_double(x, na = "-")
 NULL
 
-#' @rdname collector
-#' @export
-parse_guess <- function(x, na = c("", "NA"), locale = default_locale()) {
-  parse_vector(x, collector_guess(x, locale), na = na, locale = locale)
-}
-
-#' @rdname collector
-#' @export
-col_character <- function() {
-  collector("character")
-}
-
-#' @rdname collector
-#' @export
-parse_character <- function(x, na = c("", "NA"), locale = default_locale()) {
-  parse_vector(x, col_character(), na = na, locale = locale)
-}
-
-#' @rdname collector
-#' @export
-col_integer <- function() {
-  collector("integer")
-}
-
-#' @rdname collector
-#' @export
-parse_integer <- function(x, na = c("", "NA"), locale = default_locale()) {
-  parse_vector(x, col_integer(), na = na, locale = locale)
-}
-
-#' @rdname collector
-#' @export
-col_double <- function() {
-  collector("double")
-}
-
-#' @rdname collector
-#' @export
-parse_double <- function(x, na = c("", "NA"), locale = default_locale()) {
-  parse_vector(x, col_double(), na = na, locale = locale)
-}
-
-#' @rdname collector
-#' @export
-col_euro_double <- function() {
-  warning("Deprecated: please set locale")
-  collector("double")
-}
-
-#' @rdname collector
-#' @export
-parse_euro_double <- function(x, na = c("", "NA")) {
-  warning("Deprecated: please set locale")
-  parse_vector(x, col_double(), na = na)
-}
-
-
-#' @rdname collector
-#' @usage NULL
-#' @export
-col_numeric <- function() {
-  warning("Deprecated: please use `col_number()`")
-  collector("number")
-}
-
-#' @rdname collector
-#' @usage NULL
-#' @export
-parse_numeric <- function(x, na = c("", "NA"), locale = default_locale()) {
-  warning("Deprecated: please use `parse_number()`")
-  parse_vector(x, col_number(), na = na, locale = locale)
-}
-
-#' @rdname collector
-#' @export
-col_number <- function() {
-  collector("number")
-}
-
-#' @rdname collector
-#' @export
-parse_number <- function(x, na = c("", "NA"), locale = default_locale()) {
-  parse_vector(x, col_number(), na = na, locale = locale)
-}
-
-#' @rdname collector
-#' @export
-col_logical <- function() {
-  collector("logical")
-}
-
-#' @rdname collector
+#' @rdname parse_atomic
 #' @export
 parse_logical <- function(x, na = c("", "NA"), locale = default_locale()) {
   parse_vector(x, col_logical(), na = na, locale = locale)
 }
 
-#' @param levels Character vector providing set of allowed levels.
-#' @param ordered Is it an ordered factor?
-#' @rdname collector
+#' @rdname parse_atomic
 #' @export
-col_factor <- function(levels, ordered = FALSE) {
-  collector("factor", levels = levels, ordered = ordered)
+parse_integer <- function(x, na = c("", "NA"), locale = default_locale()) {
+  parse_vector(x, col_integer(), na = na, locale = locale)
 }
 
-#' @rdname collector
+#' @rdname parse_atomic
 #' @export
-parse_factor <- function(x, levels, ordered = FALSE, na = c("", "NA"),
-                         locale = default_locale()) {
-  parse_vector(x, col_factor(levels, ordered), na = na, locale = locale)
+parse_double <- function(x, na = c("", "NA"), locale = default_locale()) {
+  parse_vector(x, col_double(), na = na, locale = locale)
 }
 
-#' @rdname collector
+#' @rdname parse_atomic
+#' @export
+parse_character <- function(x, na = c("", "NA"), locale = default_locale()) {
+  parse_vector(x, col_character(), na = na, locale = locale)
+}
+
+#' @rdname parse_atomic
+#' @export
+col_logical <- function() {
+  collector("logical")
+}
+
+#' @rdname parse_atomic
+#' @export
+col_integer <- function() {
+  collector("integer")
+}
+
+#' @rdname parse_atomic
+#' @export
+col_double <- function() {
+  collector("double")
+}
+
+#' @rdname parse_atomic
+#' @export
+col_character <- function() {
+  collector("character")
+}
+
+#' @rdname parse_atomic
 #' @export
 col_skip <- function() {
   collector("skip")
 }
 
-#' @rdname collector
+#' Extract numbers out of an atomic vector
+#'
+#' This drops any non-numeric characters before or after the first number.
+#' The grouping mark specified by the locale is ignored inside the number.
+#'
+#' @inheritParams parse_atomic
+#' @inheritParams tokenizer_delim
+#' @inheritParams read_delim
+#' @family parser
+#' @export
+#' @examples
+#' parse_number("$1000")
+#' parse_number("1,234,567.78")
+parse_number <- function(x, na = c("", "NA"), locale = default_locale()) {
+  parse_vector(x, col_number(), na = na, locale = locale)
+}
+
+#' @rdname parse_number
+#' @export
+col_number <- function() {
+  collector("number")
+}
+
+
+#' Parse a character vector into the "best" type.
+#'
+#' \code{parse_guess()} returns the parser vector; \code{guess_parser()}
+#' returns the name of the parser. These functions use a number of heuristics
+#' to determine which type of vector is "best". Generally they try to err of
+#' the side of safety, as it's straightforward to override the parsing choice
+#' if needed.
+#'
+#' @inheritParams parse_atomic
+#' @inheritParams tokenizer_delim
+#' @inheritParams read_delim
+#' @family parser
+#' @export
+#' @examples
+#' # Logical vectors
+#' parse_guess(c("FALSE", "TRUE", "F", "T"))
+#'
+#' # Integers and doubles
+#' parse_guess(c("1","2","3"))
+#' parse_guess(c("1.6","2.6","3.4"))
+#'
+#' # Numbers containing grouping mark
+#' guess_parser("1,234,566")
+#' parse_guess("1,234,566")
+#'
+#' # ISO 8601 date times
+#' guess_parser(c("2010-10-10"))
+#' parse_guess(c("2010-10-10"))
+parse_guess <- function(x, na = c("", "NA"), locale = default_locale()) {
+  parse_vector(x, guess_parser(x, locale), na = na, locale = locale)
+}
+
+#' @rdname parse_guess
 #' @export
 col_guess <- function() {
   collector("guess")
 }
 
+#' @rdname parse_guess
+#' @export
+guess_parser <- function(x, locale = default_locale()) {
+  stopifnot(is.locale(locale))
+  collectorGuess(x, locale)
+}
+
+#' Parse a character vector into a factor
+#'
+#' @param levels Character vector providing set of allowed levels.
+#' @param ordered Is it an ordered factor?
+#' @inheritParams parse_atomic
+#' @inheritParams tokenizer_delim
+#' @inheritParams read_delim
+#' @family parser
+#' @export
+#' @examples
+#' parse_factor(c("a", "b"), letters)
+parse_factor <- function(x, levels, ordered = FALSE, na = c("", "NA"),
+                         locale = default_locale()) {
+  parse_vector(x, col_factor(levels, ordered), na = na, locale = locale)
+}
+
+#' @rdname parse_factor
+#' @export
+col_factor <- function(levels, ordered = FALSE) {
+  collector("factor", levels = levels, ordered = ordered)
+}
+
 # More complex ------------------------------------------------------------
 
 #' Parse a character vector of dates or date times.
@@ -204,7 +220,8 @@ col_guess <- function() {
 #' \enumerate{
 #'   \item Date components are specified with "\%" followed by a letter.
 #'     For example "\%Y" matches a 4 digit year, "\%m", matches a 2 digit
-#'     month and "\%d" matches a 2 digit day.
+#'     month and "\%d" matches a 2 digit day. Month and day default to \code{1},
+#'     (i.e. Jan 1st) if not present, for example if only a year is given.
 #'   \item Whitespace is any sequence of zero or more whitespace characters.
 #'   \item Any other character is matched exactly.
 #' }
@@ -225,6 +242,8 @@ col_guess <- function() {
 #'   \item Non-digits: "\%." skips one non-digit character,
 #'     "\%+" skips one or more non-digit characters,
 #'     "\%*" skips any number of non-digits characters.
+#'   \item Automatic parsers: "\%AD" parses with a flexible YMD parser,
+#'      "\%AT" parses with a flexible HMS parser.
 #'   \item Shortcuts: "\%D" = "\%m/\%d/\%y",  "\%F" = "\%Y-\%m-\%d",
 #'       "\%R" = "\%H:\%M", "\%T" = "\%H:\%M:\%S",  "\%x" = "\%y/\%m/\%d".
 #' }
@@ -247,18 +266,19 @@ col_guess <- function() {
 #' }
 #'
 #' @param x A character vector of dates to parse.
-#' @param format A format specification, as described below. If omitted,
-#'   parses dates according to the ISO8601 specification (with caveats,
-#'   as described below). Times are parsed like ISO8601 times, but also
-#'   accept an optional am/pm specification.
+#' @param format A format specification, as described below. If set to "",
+#'   date times are parsed as ISO8601, dates and times used the date and
+#'   time formats specified in the \code{\link{locale}}.
 #'
 #'   Unlike \code{\link{strptime}}, the format specification must match
 #'   the complete string.
 #' @inheritParams read_delim
+#' @inheritParams tokenizer_delim
 #' @return A \code{\link{POSIXct}} vector with \code{tzone} attribute set to
 #'   \code{tz}. Elements that could not be parsed (or did not generate valid
 #'   dates) will bes set to \code{NA}, and a warning message will inform
 #'   you of the total number of failures.
+#' @family parser
 #' @export
 #' @examples
 #' # Format strings --------------------------------------------------------
@@ -319,8 +339,20 @@ col_guess <- function() {
 #' parse_datetime("1979-10-14T1010Z", locale = us_central)
 #' # Your current time zone
 #' parse_datetime("1979-10-14T1010", locale = locale(tz = ""))
-parse_datetime <- function(x, format = "", locale = default_locale()) {
-  parse_vector(x, col_datetime(format), locale = locale)
+parse_datetime <- function(x, format = "", na = c("", "NA"), locale = default_locale()) {
+  parse_vector(x, col_datetime(format), na = na, locale = locale)
+}
+
+#' @rdname parse_datetime
+#' @export
+parse_date <- function(x, format = "", na = c("", "NA"), locale = default_locale()) {
+  parse_vector(x, col_date(format), na = na, locale = locale)
+}
+
+#' @rdname parse_datetime
+#' @export
+parse_time <- function(x, format = "", na = c("", "NA"), locale = default_locale()) {
+  parse_vector(x, col_time(format), na = na, locale = locale)
 }
 
 #' @rdname parse_datetime
@@ -331,24 +363,47 @@ col_datetime <- function(format = "") {
 
 #' @rdname parse_datetime
 #' @export
-parse_date <- function(x, format = "%Y-%m-%d", locale = default_locale()) {
-  parse_vector(x, col_date(format), locale = locale)
-}
-
-#' @rdname parse_datetime
-#' @export
-col_date <- function(format = NULL) {
+col_date <- function(format = "") {
   collector("date", format = format)
 }
 
-#' @rdname parse_datetime
-#' @export
-parse_time <- function(x, format = "", locale = default_locale()) {
-  parse_vector(x, col_time(format), locale = locale)
-}
-
 #' @rdname parse_datetime
 #' @export
 col_time <- function(format = "") {
   collector("time", format = format)
 }
+
+# Deprecated --------------------------------------------------------------
+
+#' @rdname parse_atomic
+#' @export
+#' @usage NULL
+col_euro_double <- function() {
+  warning("Deprecated: please set locale")
+  collector("double")
+}
+
+#' @rdname parse_atomic
+#' @export
+#' @usage NULL
+parse_euro_double <- function(x, na = c("", "NA")) {
+  warning("Deprecated: please set locale")
+  parse_vector(x, col_double(), na = na)
+}
+
+#' @rdname parse_atomic
+#' @usage NULL
+#' @export
+col_numeric <- function() {
+  warning("Deprecated: please use `col_number()`")
+  collector("number")
+}
+
+#' @rdname parse_atomic
+#' @usage NULL
+#' @export
+parse_numeric <- function(x, na = c("", "NA"), locale = default_locale()) {
+  warning("Deprecated: please use `parse_number()`")
+  parse_vector(x, col_number(), na = na, locale = locale)
+}
+
