@@ -50,6 +50,10 @@ public:
     column_ = Rf_lengthgets(column_, n);
   }
 
+  void clear() {
+    resize(0);
+  }
+
   void setWarnings(Warnings* pWarnings) {
     pWarnings_ = pWarnings;
   }
@@ -83,6 +87,7 @@ public:
     Collector(Rcpp::CharacterVector()),
     pEncoder_(pEncoder) {}
   void setValue(int i, const Token& t);
+  void setValue(int i, const std::string& s);
 };
 
 // Date ------------------------------------------------------------------------
@@ -93,7 +98,7 @@ class CollectorDate : public Collector {
 
 public:
   CollectorDate(LocaleInfo* pLocale, const std::string& format):
-    Collector(Rcpp::IntegerVector()),
+    Collector(Rcpp::NumericVector()),
     format_(format),
     parser_(pLocale)
   {
@@ -207,7 +212,7 @@ class CollectorTime : public Collector {
 
 public:
   CollectorTime(LocaleInfo* pLocale, const std::string& format):
-    Collector(Rcpp::IntegerVector()),
+    Collector(Rcpp::NumericVector()),
     format_(format),
     parser_(pLocale)
   {
@@ -216,7 +221,8 @@ public:
   void setValue(int i, const Token& t);
 
   Rcpp::RObject vector() {
-    column_.attr("class") = "time";
+    column_.attr("class") = Rcpp::CharacterVector::create("hms", "difftime");
+    column_.attr("units") = "secs";
     return column_;
   };
 
@@ -233,11 +239,20 @@ public:
   }
 };
 
+// Raw -------------------------------------------------------------------------
+class CollectorRaw : public Collector {
+public:
+  CollectorRaw() : Collector(Rcpp::List()) {}
+  void setValue(int i, const Token& t);
+};
+
+
 
 // Helpers ---------------------------------------------------------------------
 
-std::vector<CollectorPtr> collectorsCreate(Rcpp::ListOf<Rcpp::List> specs, LocaleInfo* pLocale, Warnings* pWarning);
+std::vector<CollectorPtr> collectorsCreate(Rcpp::ListOf<Rcpp::List> specs, LocaleInfo* pLocale);
 void collectorsResize(std::vector<CollectorPtr>& collectors, int n);
+void collectorsClear(std::vector<CollectorPtr>& collectors);
 std::string collectorGuess(Rcpp::CharacterVector input, Rcpp::List locale_);
 
 #endif
