@@ -77,11 +77,20 @@ public:
     return isComplete();
   }
 
+  bool parseLocaleTime() {
+    return parse(pLocale_->timeFormat_);
+  }
+
+  bool parseLocaleDate() {
+    return parse(pLocale_->dateFormat_);
+  }
+
   // A flexible time parser for the most common formats
   bool parseTime() {
-    if (!consumeInteger(2, &hour_))
+    if (!consumeInteger(2, &hour_, false))
+      return false;
+    if (!consumeThisChar(':'))
       return false;
-    consumeThisChar(':');
     if (!consumeInteger(2, &min_))
       return false;
     consumeThisChar(':');
@@ -94,6 +103,22 @@ public:
     return isComplete();
   }
 
+  bool parseDate() {
+    // Date: YYYY-MM-DD, YYYY/MM/DD
+    if (!consumeInteger(4, &year_))
+      return false;
+    if (!consumeThisChar('-') && !consumeThisChar('/'))
+      return false;
+    if (!consumeInteger1(2, &mon_))
+      return false;
+    if (!consumeThisChar('-') && !consumeThisChar('/'))
+      return false;
+    if (!consumeInteger1(2, &day_))
+      return false;
+
+    return isComplete();
+  }
+
   bool isComplete() {
     return dateItr_ == dateEnd_;
   }
@@ -115,7 +140,7 @@ public:
         continue;
       }
 
-      // Any other characters much much exactly.
+      // Any other characters must much exactly.
       if (*formatItr != '%') {
         if (!consumeThisChar(*formatItr))
           return false;
@@ -137,7 +162,7 @@ public:
         year_ += (year_ < 69) ? 2000 : 1900;
         break;
       case 'm': // month
-        if (!consumeInteger1(2, &mon_))
+        if (!consumeInteger1(2, &mon_, false))
           return false;
         break;
       case 'b': // abbreviated month name
@@ -149,7 +174,7 @@ public:
           return false;
         break;
       case 'd': // day
-        if (!consumeInteger1(2, &day_))
+        if (!consumeInteger1(2, &day_, false))
           return false;
         break;
       case 'e': // day with optional leading space
@@ -157,9 +182,17 @@ public:
           return false;
         break;
       case 'H': // hour
-        if (!consumeInteger(2, &hour_))
+        if (!consumeInteger(2, &hour_, false))
           return false;
         break;
+      case 'I': // hour
+        if (!consumeInteger(2, &hour_, false))
+          return false;
+        if (hour_ < 1 || hour_ > 12) {
+          return false;
+        }
+        hour_ %= 12;
+        break;
       case 'M': // minute
         if (!consumeInteger(2, &min_))
           return false;
@@ -206,6 +239,24 @@ public:
         consumeNonDigits();
         break;
 
+      case 'A': // auto date / time
+        if (formatItr + 1 == formatEnd)
+          Rcpp::stop("Invalid format: %%A must be followed by another letter");
+        formatItr++;
+        switch(*formatItr) {
+        case 'D':
+          if (!parseDate())
+            return false;
+          break;
+        case 'T':
+          if (!parseTime())
+            return false;
+          break;
+        default:
+          Rcpp::stop("Invalid %%A auto parser");
+        }
+        break;
+
         // Compound formats
       case 'D':
         parse("%m/%d/%y");
@@ -289,17 +340,20 @@ private:
     return false;
   }
 
-  inline bool consumeInteger(int n, int* pOut) {
+  inline bool consumeInteger(int n, int* pOut, bool exact = true) {
     if (dateItr_ == dateEnd_ || *dateItr_ == '-' || *dateItr_ == '+')
       return false;
 
+    const char* start = dateItr_;
     const char* end = std::min(dateItr_ + n, dateEnd_);
-    return parseInt(dateItr_, end, *pOut);
+    bool ok = parseInt(dateItr_, end, *pOut);
+
+    return ok && (!exact || (dateItr_ - start) == n);
   }
 
   // Integer indexed from 1 (i.e. month and date)
-  inline bool consumeInteger1(int n, int* pOut) {
-    if (!consumeInteger(n, pOut))
+  inline bool consumeInteger1(int n, int* pOut, bool exact = true) {
+    if (!consumeInteger(n, pOut, exact))
       return false;
 
     (*pOut)--;
@@ -421,8 +475,8 @@ private:
 
   void reset() {
     year_ = -1;
-    mon_ = -1;
-    day_ = -1;
+    mon_ = 0;
+    day_ = 0;
     hour_ = 0;
     min_ = 0;
     sec_ = 0;
