@@ -3,7 +3,7 @@
 
 #include "boost.h"
 
-struct DecimalCommaPolicy : public boost::spirit::qi::real_policies<double> {
+struct DecimalCommaPolicy : public boost::spirit::qi::real_policies<long double> {
   template <typename Iterator>
   static bool parse_dot(Iterator& first, Iterator const& last) {
     if (first == last || *first != ',')
@@ -19,10 +19,10 @@ inline bool parseDouble(const char decimalMark, Iterator& first, Iterator& last,
 
   if (decimalMark == '.') {
     return boost::spirit::qi::parse(first, last,
-      boost::spirit::qi::double_, res);
+      boost::spirit::qi::long_double, res);
   } else if (decimalMark == ',') {
     return boost::spirit::qi::parse(first, last,
-      boost::spirit::qi::real_parser<double, DecimalCommaPolicy>(), res);
+      boost::spirit::qi::real_parser<long double, DecimalCommaPolicy>(), res);
   } else {
     return false;
   }
@@ -36,6 +36,8 @@ enum NumberState {
 };
 
 
+// First and last are updated to point to first/last successfully parsed
+// character
 template <typename Iterator, typename Attr>
 inline bool parseNumber(char decimalMark, char groupingMark, Iterator& first,
                         Iterator& last, Attr& res) {
@@ -51,20 +53,27 @@ inline bool parseNumber(char decimalMark, char groupingMark, Iterator& first,
 
   double sum = 0, denom = 1;
   NumberState state = STATE_INIT;
+  bool seenNumber = false;
+  double sign = 1.0;
 
   Iterator cur = first;
   for(; cur != last; ++cur) {
+    if (state == STATE_FIN)
+      break;
+
     switch(state) {
     case STATE_INIT:
       if (*cur == '-') {
         state = STATE_LHS;
+        sign = -1.0;
       } else if (*cur == decimalMark) {
         state = STATE_RHS;
       } else if (*cur >= '0' && *cur <= '9') {
+        seenNumber = true;
         state = STATE_LHS;
         sum = *cur - '0';
       } else {
-        return false;
+        goto end;
       }
       break;
     case STATE_LHS:
@@ -73,44 +82,42 @@ inline bool parseNumber(char decimalMark, char groupingMark, Iterator& first,
       } else if (*cur == decimalMark) {
         state = STATE_RHS;
       } else if (*cur >= '0' && *cur <= '9') {
+        seenNumber = true;
         sum *= 10;
         sum += *cur - '0';
-      } else if (*cur == '-') {
-        return false;
       } else {
-        state = STATE_FIN;
-        break;
+        goto end;
       }
       break;
     case STATE_RHS:
       if (*cur == groupingMark) {
         // do nothing
-      } else if (*cur == decimalMark) {
-        return false;
       } else if (*cur >= '0' && *cur <= '9') {
+        seenNumber = true;
         denom *= 10;
         sum += (*cur - '0') / denom;
       } else {
-        state = STATE_FIN;
+        goto end;
       }
       break;
     case STATE_FIN:
-      last = cur++;
-      res = sum;
-      return true;
+      goto end;
     }
   }
 
-  // Hit the end of the string, so must be done
+end:
+
+  // Set last to point to final character used
   last = cur;
-  res = sum;
-  return true;
+
+  res = sign * sum;
+  return seenNumber;
 }
 
 
 template <typename Iterator, typename Attr>
 inline bool parseInt(Iterator& first, Iterator& last, Attr& res) {
-  return  boost::spirit::qi::parse(first, last, boost::spirit::qi::int_, res);
+  return boost::spirit::qi::parse(first, last, boost::spirit::qi::int_, res);
 }
 
 #endif
