@@ -18,19 +18,22 @@ TokenizerPtr Tokenizer::create(List spec) {
     bool trimWs = as<bool>(spec["trim_ws"]);
     bool escapeDouble = as<bool>(spec["escape_double"]);
     bool escapeBackslash = as<bool>(spec["escape_backslash"]);
+    bool quotedNA = as<bool>(spec["quoted_na"]);
 
     return TokenizerPtr(new
-      TokenizerDelim(delim, quote, na, comment, trimWs, escapeBackslash, escapeDouble)
+      TokenizerDelim(delim, quote, na, comment, trimWs, escapeBackslash, escapeDouble, quotedNA)
     );
   } else if (subclass == "tokenizer_fwf") {
     std::vector<int>
       begin = as<std::vector<int> >(spec["begin"]),
       end = as<std::vector<int> >(spec["end"]);
     std::vector<std::string> na = as<std::vector<std::string> >(spec["na"]);
+    std::string comment = as<std::string>(spec["comment"]);
 
-    return TokenizerPtr(new TokenizerFwf(begin, end, na));
+    return TokenizerPtr(new TokenizerFwf(begin, end, na, comment));
   } else if (subclass == "tokenizer_line") {
-    return TokenizerPtr(new TokenizerLine());
+    std::vector<std::string> na = as<std::vector<std::string> >(spec["na"]);
+    return TokenizerPtr(new TokenizerLine(na));
   } else if (subclass == "tokenizer_log") {
     return TokenizerPtr(new TokenizerLog());
   }
